function [Xb,x_b,Xa,x_a] = kf_lor3v(x0_t,t,tobs,y,H,R,rho,M)
 % [Xb,x_b,Xa,x_a] = kf_lor3v(x0_t,t,tobs,y,R,rho,m,metana,obs_order,Bout)
 % Data assimilation for Lorenz 1963 using ETKF
 % inputs    - x0_t, the real initial position [1,N]
 %           - t, time array of the model
 %           - tobs, time array of the observations
 %           - y, the observations
 %           - H, observation matrix
 %           - R, the observational error covariance matrix 
 %           - rho, inflation for P. notice we multiply (1+rho)*Xpert or P*(1+rho)^2
 %           - M, the ensemble size
 % outputs   - Xb, the background ensemble 3D array [time,vars,members]
 %           - xb, background mean
 %           - Xa, the background ensemble 3D array [time,vars,members]
 %           - xa, the analysis array

%% General settings
 % number of variables
 N = length(x0_t);
 % for the true time
 tstep_truth = t(2)-t(1);
 % and for the analysis (we assimilate everytime we get observations)
 tstep_obs = tobs(2)-tobs(1);
 % the ratio
 o2t = tstep_obs/tstep_truth;
 % precreate the arrays for background and analysis
 Xb = NaN(length(t),N,M);
 Xa = NaN(length(t),N,M);

%% For the original background ensemble
 % Two options: fixed and random
 back0 = 'fixed';
 if strcmp(back0,'fixed') % fixed initial conditions for our ensemble, ad hoc
  xaux = zeros(N,M);
  desv = R(1,1);
  for j=1:N;   xaux(j,:) = linspace(x0_t(1,j)-sqrt(desv),x0_t(1,j)+sqrt(desv),M);  end
  for jm=1:M;  for jn=1:N;     Xb(1,jn,jm) = xaux(jn,jm);      end;        end
 end

 if strcmp(back0,'random') % random initial conditions for our ensemble, let's perturb the real x0 with with the magnitudes of R
  for jm=1:M;  for jn=1:N;     Xb(1,jn,jm) = x0_t(1,jn) + sqrt(R(jn,jn))*randn(1);     end;    end
 end

 % since we don't have obs at t=0 the first analysis is the same as background
 Xa(1,:,:) = Xb(1,:,:);

%% the following cycle contains evolution and assimilation
 % for all the time steps
 for j=1:length(tobs)-1
  % first we evolve the ensemble members
  % evolve from analysis!
  xold = Xa((j-1)*o2t+1,:,:); % [1,n,m]       
  xold = squeeze(xold); % [n,m]
  % time goes forward
  xnew = evolvemembers(xold,tstep_truth,o2t); % needs [n,m] arrays, gives [o2t+1,n,m]!
  % the new background
  Xb((j-1)*o2t+2:j*o2t+1,:,:) = xnew(2:o2t+1,:,:); % [o2t,n,m]
  Xa((j-1)*o2t+2:j*o2t+1,:,:) = xnew(2:o2t+1,:,:); % [o2t,n,m]
  % the assimilation
  Xa_aux = etkf(Xb(j*o2t+1,:,:),y(j+1,:),H,R,rho); % gets a [n,m] matrix
  Xa_aux = shiftdim(Xa_aux,-1); % [1,n,m]
  Xa(j*o2t+1,:,:) = Xa_aux;
  %j
 end

 % the background and analysis mean
 x_b = mean(Xb,3); % [t2,n,m] -> [t2,n]
 x_a = mean(Xa,3); % [t1,n,m] -> [t1,n]
end


%% ________________________________________________
function xnew = evolvemembers(xold,tstep_truth,o2t)
 % xnew = evolvemembersSF(xold,tstep_truth,o2t)
 % inputs:  - xold, a [n,m] matrix of initial conditions (in every step) for the m members and n variables
 %          - tstep_truth, the time step used in the nature run
 %          - o2t, frequency of observtions in time steps
 % outputs: - xnew, a [o2t+1,n,m] array with the analysis
 t_anal = o2t*tstep_truth;
 tam = size(xold); %[n,m]
 xnew = NaN(o2t+1,tam(1),tam(2)); %[o2t+1,n,m]
 
 for j=1:tam(2)
  [taux,xaux] = lorenz3v(xold(:,j)',t_anal); %gives a vector with size [o2t+1,n]
  xnew(:,:,j) = xaux;
 end
end

%%
function Xa = etkf(Xb,y,H,R,rho)
 % performs the analysis using ETKF
 % inputs: - Xb, the ensemble background [1,n,m]
 %         - y, the observations [1,3]
 %         - H, the observation matrix
 %         - R, the obs error covariance
 %         - rho, inflation for P. notice we multiply (1+rho)*Xpert or P*(1+rho)^2
 % Output: - Xa, the full analysis ensemble 
 %% General settings
 [p,n,m] = size(Xb); % p is never used, it's actually 1
 y = y'; % [1,l] -> [n,l]

 % the background information
 Xb = squeeze(Xb); %[1,n,m]->[n,m]
 xb_bar = mean(Xb,2);
 Xb_pert = Xb-xb_bar*ones(1,m); %[n,m]

 yb_bar = H*xb_bar;
 Yb_pert = H*Xb_pert;
    
 Pa_tilde = inv(((m-1)/(1+rho)^2)*eye(m,m)+Yb_pert'*inv(R)*Yb_pert); %inflation here
 Wa = sqrtm((m-1)*Pa_tilde); % attention: this is a symmetric matrix square root
 wa = Pa_tilde*Yb_pert'*inv(R)*(y-yb_bar);
  
 Xa_pert = Xb_pert*Wa;   
 xa_bar = xb_bar + Xb_pert*wa;
 Xa = Xa_pert + xa_bar*ones(1,m);
end 



